@@ -22,15 +22,11 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
 
-try:
-    from colorlog import ColoredFormatter
-except:
-    ColoredFormatter = None
-
 import gconf
 import gobject
 import gtk
 import json
+import keybinder
 import logging
 import logging.config
 import os
@@ -38,6 +34,7 @@ import platform
 import pygtk
 import subprocess
 import sys
+import uuid
 import xdg.Exceptions
 
 from urllib import quote_plus
@@ -47,7 +44,11 @@ from xdg.DesktopEntry import DesktopEntry
 from xml.sax.saxutils import escape as xml_escape
 
 import guake.notifier
-import keybinder
+
+try:
+    from colorlog import ColoredFormatter
+except:
+    ColoredFormatter = None
 
 from guake.about import AboutDialog
 from guake.common import _
@@ -109,6 +110,9 @@ GDK_WINDOW_STATE_ICONIFIED = 2
 GDK_WINDOW_STATE_STICKY = 8
 GDK_WINDOW_STATE_ABOVE = 32
 
+# Transparency max level (should be always 100)
+MAX_TRANSPARENCY = 100
+
 
 log = logging.getLogger(__name__)
 
@@ -168,9 +172,8 @@ class Guake(SimpleGladeApp):
         self.preventHide = False
 
         # trayicon! Using SVG handles better different OS trays
-        # img = pixmapfile('guake-tray.svg')
-        # trayicon!
-        img = pixmapfile('guake-tray.png')
+        img = pixmapfile('guake-tray.svg')
+
         try:
             import appindicator
         except ImportError:
@@ -250,6 +253,12 @@ class Guake(SimpleGladeApp):
         # holds the timestamp of the previous show/hide action
         self.prev_showhide_time = 0
 
+        # holds transparency level
+        self.transparency = 0
+
+        # Controls the transparency state needed for function accel_toggle_transparency
+        self.transparency_toggled = False
+
         # load cumstom command menu and menuitems from config file
         self.custom_command_menuitem = None
         self.load_custom_commands()
@@ -309,9 +318,6 @@ class Guake(SimpleGladeApp):
         self.window.connect('delete-event', destroy)
         self.window.connect('window-state-event', window_event)
 
-        # Flag to completely disable losefocus hiding
-        self.disable_losefocus_hiding = False
-
         # this line is important to resize the main window and make it
         # smaller.
         self.window.set_geometry_hints(min_width=1, min_height=1)
@@ -532,6 +538,23 @@ class Guake(SimpleGladeApp):
             terminal.feed_child(command)
             break
 
+    def execute_command_by_uuid(self, tab_uuid, command):
+        """Execute the `command' in the tab whose terminal has the `tab_uuid' uuid
+        """
+        if command[-1] != '\n':
+            command += '\n'
+        try:
+            tab_uuid = uuid.UUID(tab_uuid)
+            tab_index, = (index for index, t in enumerate(
+                self.notebook.iter_terminals()) if t.get_uuid() == tab_uuid)
+            self.tabs.get_children()[tab_index]
+        except ValueError:
+            pass
+        else:
+            terminals = self.notebook.get_terminals_for_tab(tab_index)
+            for current_vte in terminals:
+                current_vte.feed_child(command)
+
     def on_resizer_drag(self, widget, event):
         """Method that handles the resize drag. It does not actuall
         moves the main window. It just set the new window size in
@@ -560,7 +583,7 @@ class Guake(SimpleGladeApp):
         """Hides terminal main window when it loses the focus and if
         the window_losefocus gconf variable is True.
         """
-        if self.disable_losefocus_hiding or self.showing_context_menu:
+        if self.showing_context_menu:
             return
 
         if self.prompt_dialog is not None:
@@ -946,7 +969,7 @@ class Guake(SimpleGladeApp):
                          '/org/compiz/profiles/unity/plugins/unityshell/launcher-hide-mode']))
                     unity_dock = int(subprocess.check_output(
                         ['/usr/bin/dconf', 'read',
-                         '/org/compiz/profiles/unity/plugins/unityshell/icon-size']))
+                         '/org/compiz/profiles/unity/plugins/unityshell/icon-size']) or "48")
                     found = True
                 except:
                     # in case of error, just ignore it, 'found' will not be set to True and so
@@ -1029,6 +1052,7 @@ class Guake(SimpleGladeApp):
         self.client.notify(KEY('/general/history_size'))
         self.client.notify(KEY('/general/show_resizer'))
         self.client.notify(KEY('/general/use_vte_titles'))
+        self.client.notify(KEY('/general/max_tab_name_length'))
         self.client.notify(KEY('/general/quick_open_enable'))
         self.client.notify(KEY('/general/quick_open_command_line'))
         self.client.notify(KEY('/style/cursor_shape'))
@@ -1119,6 +1143,8 @@ class Guake(SimpleGladeApp):
         """Callback to increase transparency.
         """
         transparency = self.client.get_int(KEY('/style/background/transparency'))
+        if transparency >= MAX_TRANSPARENCY:
+            return True
         self.client.set_int(KEY('/style/background/transparency'), int(transparency) + 2)
         return True
 
@@ -1126,9 +1152,23 @@ class Guake(SimpleGladeApp):
         """Callback to decrease transparency.
         """
         transparency = self.client.get_int(KEY('/style/background/transparency'))
+        if transparency <= 0:
+            return True
         self.client.set_int(KEY('/style/background/transparency'), int(transparency) - 2)
         return True
 
+    def accel_toggle_transparency(self, *args):
+        """Callback to toggle transparency.
+        """
+        if self.transparency_toggled:
+            self.client.set_int(KEY('/style/background/transparency'), int(self.transparency))
+            self.transparency_toggled = False
+            return True
+        self.transparency = self.client.get_int(KEY('/style/background/transparency'))
+        self.client.set_int(KEY('/style/background/transparency'), MAX_TRANSPARENCY)
+        self.transparency_toggled = True
+        return True
+
     def accel_add(self, *args):
         """Callback to add a new tab. Called by the accel key.
         """
@@ -1177,6 +1217,11 @@ class Guake(SimpleGladeApp):
 
         return callback
 
+    def accel_switch_tab_last(self, *args):
+        last_tab = self.notebook.get_n_pages() - 1
+        self.notebook.set_current_page(last_tab)
+        return True
+
     def accel_rename_current_tab(self, *args):
         """Callback to show the rename tab dialog. Called by the accel
         key.
@@ -1222,8 +1267,11 @@ class Guake(SimpleGladeApp):
         """Callback toggle whether the window should hide when it loses
         focus. Called by the accel key.
         """
-        # use temporary setting -- don't change conf key
-        self.disable_losefocus_hiding = not self.disable_losefocus_hiding
+
+        if self.client.get_bool(KEY('/general/window_losefocus')):
+            self.client.set_bool(KEY('/general/window_losefocus'), False)
+        else:
+            self.client.set_bool(KEY('/general/window_losefocus'), True)
         return True
 
     def fullscreen(self):
@@ -1275,7 +1323,11 @@ class Guake(SimpleGladeApp):
         tab = self.tabs.get_children()[page]
         # if tab has been renamed by user, don't override.
         if not getattr(tab, 'custom_label_set', False):
-            tab.set_label(vte.get_window_title())
+            vte_title = vte.get_window_title()
+            max_name_length = self.client.get_int(KEY("/general/max_tab_name_length"))
+            if len(vte_title) > max_name_length and max_name_length is not 0:
+                vte_title = vte_title[:max_name_length]
+            tab.set_label(vte_title)
 
     def on_rename_current_tab_activate(self, *args):
         """Shows a dialog to rename the current tab.
@@ -1304,16 +1356,24 @@ class Guake(SimpleGladeApp):
         entry.reparent(vbox)
 
         # don't hide on lose focus until the rename is finished
-        current_hide_setting = self.disable_losefocus_hiding
-        self.disable_losefocus_hiding = True
+        self.preventHide = True
         response = dialog.run()
-        self.disable_losefocus_hiding = current_hide_setting
+        self.preventHide = False
 
         if response == gtk.RESPONSE_ACCEPT:
+            max_name_length = self.client.get_int(KEY("/general/max_tab_name_length"))
             new_text = entry.get_text()
+            if len(new_text) > max_name_length and max_name_length is not 0:
+                new_text = new_text[:max_name_length]
+
             self.selected_tab.set_label(new_text)
             # if user sets empty name, consider he wants default behavior.
             setattr(self.selected_tab, 'custom_label_set', bool(new_text))
+
+            # holds custom label name of the tab,
+            # we need this to restore the name if the max length is changed
+            setattr(self.selected_tab, 'custom_label_text', new_text)
+
             # trigger titling handler in case that custom label has been reset
             current_vte = self.notebook.get_current_terminal()
             current_vte.emit('window-title-changed')
@@ -1376,6 +1436,23 @@ class Guake(SimpleGladeApp):
         pagepos = self.notebook.get_current_page()
         self.delete_tab(pagepos)
 
+    def rename_tab_uuid(self, tab_uuid, new_text):
+        """Rename an already added tab by its UUID
+        """
+        try:
+            tab_uuid = uuid.UUID(tab_uuid)
+            tab_index, = (index for index, t in enumerate(
+                self.notebook.iter_terminals()) if t.get_uuid() == tab_uuid)
+            tab = self.tabs.get_children()[tab_index]
+        except ValueError:
+            pass
+        else:
+            tab.set_label(new_text)
+            setattr(tab, 'custom_label_set', new_text != "-")
+            terminals = self.notebook.get_terminals_for_tab(tab_index)
+            for current_vte in terminals:
+                current_vte.emit('window-title-changed')
+
     def rename_tab(self, tab_index, new_text):
         """Rename an already added tab by its index.
         """
@@ -1421,7 +1498,7 @@ class Guake(SimpleGladeApp):
                     directory = cwd
         return directory
 
-    def get_fork_params(self, default_params=None):
+    def get_fork_params(self, default_params=None, box=None):
         """Return all parameters to be passed to the fork_command
         method of a vte terminal. Params returned can be expanded by
         the `params' parameter that receive a dictionary.
@@ -1451,10 +1528,10 @@ class Guake(SimpleGladeApp):
         # Environment variables are not actually parameters but they
         # need to be set before calling terminal.fork_command()
         # method. So I found this place good to do it.
-        self.update_proxy_vars()
+        self.update_proxy_vars(box)
         return params
 
-    def update_proxy_vars(self):
+    def update_proxy_vars(self, box=None):
         """This method updates http{s,}_proxy environment variables
         with values found in gconf.
         """
@@ -1483,6 +1560,10 @@ class Guake(SimpleGladeApp):
                 os.environ['http_proxy'] = 'http://%s:%d' % (host, port)
                 os.environ['https_proxy'] = 'http://%s:%d' % (
                     ssl_host, ssl_port)
+        if box:
+            os.environ['GUAKE_TAB_UUID'] = str(box.terminal.get_uuid())
+        else:
+            del os.environ['GUAKE_TAB_UUID']
 
     def add_tab(self, directory=None):
         """Adds a new tab to the terminal notebook.
@@ -1520,7 +1601,7 @@ class Guake(SimpleGladeApp):
         if isinstance(directory, basestring):
             default_params['directory'] = directory
 
-        final_params = self.get_fork_params(default_params)
+        final_params = self.get_fork_params(default_params, box)
         pid = box.terminal.fork_command(**final_params)
         if libutempter is not None:
             # After the fork_command we add this new tty to utmp !
@@ -1532,7 +1613,6 @@ class Guake(SimpleGladeApp):
         label = box.terminal.get_window_title() or _("Terminal")
         tabs = self.tabs.get_children()
         parent = tabs and tabs[0] or None
-
         bnt = gtk.RadioButton(group=parent, label=label, use_underline=False)
         bnt.set_property('can-focus', False)
         bnt.set_property('draw-indicator', False)
@@ -1561,6 +1641,8 @@ class Guake(SimpleGladeApp):
         if self.is_fullscreen:
             self.fullscreen()
 
+        return str(box.terminal.get_uuid())
+
     def save_tab(self, directory=None):
         self.preventHide = True
         current_term = self.notebook.get_current_terminal()
@@ -1690,6 +1772,13 @@ class Guake(SimpleGladeApp):
         self.select_tab(0)
         self.select_tab(pos)
 
+    def get_selected_uuidtab(self):
+        """Returns the uuid of the current selected terminal
+        """
+        pagepos = self.notebook.get_current_page()
+        terminals = self.notebook.get_terminals_for_tab(pagepos)
+        return str(terminals[0].get_uuid())
+
     def select_current_tab(self, notebook, user_data, page):
         """When current self.notebook page is changed, the tab bar
         made with radio buttons must be updated and this method does
