@@ -123,6 +123,8 @@ HOTKEYS = [
                'label': 'Go to ninth tab'},
               {'key': LKEY('switch_tab10'),
                'label': 'Go to tenth tab'},
+              {'key': LKEY('switch_tab_last'),
+               'label': 'Go to last tab'},
               ]},
 
     {'label': 'Appearance',
@@ -676,8 +678,8 @@ class PrefsDialog(SimpleGladeApp):
         """Updates the color buttons with the given palette
         """
         palette = palette.split(':')
-        for i in range(len(palette)):
-            color = gtk.gdk.color_parse(palette[i])
+        for i, pal in enumerate(palette):
+            color = gtk.gdk.color_parse(pal)
             self.get_widget('palette_%d' % i).set_color(color)
 
     def reload_erase_combos(self, btn=None):
